@@ -68,7 +68,7 @@ except ImportError:
             _ = self.request.getText
             from MoinMoin.parser.text import Parser as TextParser
             self.request.write(formatter.sysmsg(1) +
-                               formatter.rawHTML(_('Rendering of reStructured text is not possible, please install Docutils.', formatted=False)) +
+                               formatter.rawHTML(_('Rendering of reStructured text is not possible, please install Docutils.')) +
                                formatter.sysmsg(0))
             TextParser(self.raw, self.request).format(formatter)
 
@@ -170,9 +170,30 @@ class MoinWriter(html4css1.Writer):
                 setattr(self, attr, getattr(visitor, attr))
         self.output = html_escape_unicode(visitor.astext())
 
+# mark quickhelp as translatable
+_ = lambda x: x
+
 class Parser:
     caching = 1
     Dependencies = Dependencies # copy dependencies from module-scope
+    quickhelp = _("""\
+{{{
+Emphasis: *italic* **bold** ``monospace``
+
+Headings: Heading 1  Heading 2  Heading 3
+          =========  ---------  ~~~~~~~~~
+
+Horizontal rule: ----
+
+Links: TrailingUnderscore_ `multi word with backticks`_ external_
+
+.. _external: http://external-site.example.org/foo/
+
+Lists: * bullets; 1., a. numbered items.
+}}}
+(!) For more help, see the
+[[http://docutils.sourceforge.net/docs/user/rst/quickref.html|reStructuredText Quick Reference]].
+""")
 
     def __init__(self, raw, request, **kw):
         self.raw = raw
@@ -357,8 +378,7 @@ class MoinTranslator(html4css1.HTMLTranslator):
                     self.wiki_text = self.fixup_wiki_formatting(self.wiki_text)
                     self.add_wiki_markup()
                 # Attachment exists, just get a link to it.
-                node['refuri'] = AttachFile.getAttachUrl(self.request.page.page_name,
-                        link, self.request)
+                node['refuri'] = AttachFile.getAttachUrl(self.request.page.page_name, link, self.request)
                 if not [i for i in node.children if i.__class__ == docutils.nodes.image]:
                     node['classes'].append(prefix)
             elif prefix == 'wiki':
@@ -385,7 +405,7 @@ class MoinTranslator(html4css1.HTMLTranslator):
                 if '#' in refuri:
                     pagename, anchor = rsplit(refuri, '#', 1)
                 page = Page(self.request, wikiutil.AbsPageName(self.formatter.page.page_name, pagename))
-                node['refuri'] = page.url(self.request, anchor=anchor, relative=False)
+                node['refuri'] = page.url(self.request, anchor=anchor)
                 if not page.exists():
                     node['classes'].append('nonexistent')
         html4css1.HTMLTranslator.visit_reference(self, node)
@@ -417,8 +437,7 @@ class MoinTranslator(html4css1.HTMLTranslator):
                 self.add_wiki_markup()
             # Attachment exists, get a link to it.
             # create the url
-            node['uri'] = AttachFile.getAttachUrl(self.request.page.page_name,
-                    attach_name, self.request, addts=1)
+            node['uri'] = AttachFile.getAttachUrl(self.request.page.page_name, attach_name, self.request, addts=1)
             if not node.hasattr('alt'):
                 node['alt'] = node.get('name', uri)
         html4css1.HTMLTranslator.visit_image(self, node)
@@ -481,11 +500,10 @@ class MoinTranslator(html4css1.HTMLTranslator):
 
     # Admonitions are handled here -=- tmacam
     def create_admonition_functor(self, admotion_class):
-        tag_class = 'admonition_' + admotion_class
         def visit_func(self, node):
             self.wiki_text = ''
             self.request.write(self.formatter.div(1,
-                                                  attr={'class': tag_class},
+                                                  attr={'class': admotion_class},
                                                   allowed_attrs=[]))
             self.body.append(self.wiki_text)
         def depart_func(self, node):
@@ -560,7 +578,7 @@ class MoinDirectives:
         if self.num_includes < self.max_includes:
             self.num_includes += 1
         else:
-            lines = [_("**Maximum number of allowed includes exceeded**", formatted=False)]
+            lines = [_("**Maximum number of allowed includes exceeded**")]
             state_machine.insert_input(lines, 'MoinDirectives')
             return
 
@@ -568,7 +586,7 @@ class MoinDirectives:
             pagename = content[0]
             page = Page(page_name=pagename, request=self.request)
             if not self.request.user.may.read(pagename):
-                lines = [_("**You are not allowed to read the page: %s**", formatted=False) % (pagename, )]
+                lines = [_("**You are not allowed to read the page: %s**") % (pagename, )]
             else:
                 if page.exists():
                     text = page.get_raw_body()
@@ -577,7 +595,7 @@ class MoinDirectives:
                     if lines[0].startswith("#format"):
                         del lines[0]
                 else:
-                    lines = [_("**Could not find the referenced page: %s**", formatted=False) % (pagename, )]
+                    lines = [_("**Could not find the referenced page: %s**") % (pagename, )]
             # Insert the text from the included document and then continue parsing
             state_machine.insert_input(lines, 'MoinDirectives')
         return
@@ -615,3 +633,4 @@ class MoinDirectives:
 if ErrorParser: # fixup in case of missing docutils
     Parser = ErrorParser
 
+del _
