@@ -1,7 +1,7 @@
 # -*- coding: iso-8859-1 -*-
 """
     MoinMoin - search results processing
-    
+
     @copyright: 2005 MoinMoin:FlorianFesti,
                 2005 MoinMoin:NirSoffer,
                 2005 MoinMoin:AlexanderSchremmer,
@@ -21,7 +21,7 @@ from MoinMoin.Page import Page
 
 class Match(object):
     """ Base class for all Matches (found pieces of pages).
-    
+
     This class represents a empty True value as returned from negated searches.
     """
     # Default match weight
@@ -75,7 +75,7 @@ class TextMatch(Match):
 
 class TitleMatch(Match):
     """ Represents a match in the page title
-    
+
     Has more weight than a match in the page content.
     """
     # Matches in titles are much more important in wikis. This setting
@@ -128,7 +128,7 @@ class FoundPage:
 
         @param unique: return only unique matches (bool)
         @param sort: match attribute to sort by (string)
-        @param type: type of match to return (Match or sub class) 
+        @param type: type of match to return (Match or sub class)
         @rtype: list
         @return: list of matches
         """
@@ -278,7 +278,7 @@ class SearchResults:
             formatter.paragraph(1, attr={'class': 'searchstats'}),
             _("Results %(bs)s%(hitsFrom)d - %(hitsTo)d%(be)s "
                     "of %(aboutHits)s %(bs)s%(hits)d%(be)s results out of "
-                    "about %(pages)d pages.", formatted=False) %
+                    "about %(pages)d pages.") %
                 {'aboutHits': self.estimated_hits[0],
                     'hits': self.estimated_hits[1], 'pages': self.pages,
                     'hitsFrom': hitsFrom + 1,
@@ -289,7 +289,7 @@ class SearchResults:
             u' (%s %s)' % (''.join([formatter.strong(1),
                 formatter.text("%.2f" % self.elapsed),
                 formatter.strong(0)]),
-                formatter.text(_("seconds", formatted=False))),
+                formatter.text(_("seconds"))),
             formatter.paragraph(0),
             ]
         return ''.join(output)
@@ -304,6 +304,7 @@ class SearchResults:
         @param numbered: use numbered list for display
         @param paging: toggle paging
         @param hitsFrom: current position in the hits
+        @param hitsInfo: toggle hits info line
         @rtype: unicode
         @return formatted page list
         """
@@ -332,7 +333,7 @@ class SearchResults:
                 if page.attachment:
                     querydict = {
                         'action': 'AttachFile',
-                        'do': 'get',
+                        'do': 'view',
                         'target': page.attachment,
                     }
                 elif page.page.rev and page.page.rev != page.page.getRevList()[0]:
@@ -402,7 +403,7 @@ class SearchResults:
             write(f.definition_list(1))
 
             if paging:
-                hitsTo = hitsFrom+request.cfg.search_results_per_page
+                hitsTo = hitsFrom + request.cfg.search_results_per_page
                 displayHits = self.hits[hitsFrom:hitsTo]
             else:
                 displayHits = self.hits
@@ -415,7 +416,7 @@ class SearchResults:
                     fmt_context = ""
                     querydict = {
                         'action': 'AttachFile',
-                        'do': 'get',
+                        'do': 'view',
                         'target': page.attachment,
                     }
                 elif page.page_name.startswith('FS/'): # XXX FS hardcoded
@@ -497,7 +498,7 @@ class SearchResults:
                 if matches[j].start >= start:
                     break
 
-            # Add all matches in context and the text between them 
+            # Add all matches in context and the text between them
             while True:
                 match = matches[j]
                 # Ignore matches behind the current position
@@ -542,7 +543,7 @@ class SearchResults:
 
         This function is needed only because we don't have yet a weight
         algorithm for page text matches.
-        
+
         Try to find the first match in the page text. If we can't find
         one, we return the first match and start=0.
 
@@ -670,8 +671,7 @@ class SearchResults:
         querydict = wikiutil.parseQueryString(self.request.query_string)
         def page_url(n):
             querydict.update({'from': n * hitsPerPage})
-            return self.request.page.url(self.request, querydict,
-                    escape=0, relative=False)
+            return self.request.page.url(self.request, querydict, escape=0)
 
         pages = hitsNum // hitsPerPage
         remainder = hitsNum % hitsPerPage
@@ -685,7 +685,7 @@ class SearchResults:
         if cur_page > 0:
             textlinks.append(''.join([
                 f.url(1, href=page_url(cur_page-1)),
-                f.text(_('Previous', formatted=False)),
+                f.text(_('Previous')),
                 f.url(0)
             ]))
         else:
@@ -704,10 +704,10 @@ class SearchResults:
             ]) for i in page_range])
 
         # next page available
-        if cur_page < pages-1:
+        if cur_page < pages - 1:
             textlinks.append(''.join([
                 f.url(1, href=page_url(cur_page+1)),
-                f.text(_('Next', formatted=False)),
+                f.text(_('Next')),
                 f.url(0)
             ]))
         else:
@@ -741,10 +741,10 @@ class SearchResults:
         size_str = '%.1fk' % (p.size()/1024.0)
         revisions = p.getRevList()
         if len(revisions) and rev == revisions[0]:
-            rev_str = '%s: %d (%s)' % (_('rev', formatted=False), rev, _('current', formatted=False))
+            rev_str = '%s: %d (%s)' % (_('rev'), rev, _('current'))
         else:
-            rev_str = '%s: %d' % (_('rev', formatted=False), rev, )
-        lastmod_str = _('last modified: %s', formatted=False) % p.mtime_printable(request)
+            rev_str = '%s: %d' % (_('rev'), rev, )
+        lastmod_str = _('last modified: %s') % p.mtime_printable(request)
 
         result = f.paragraph(1, attr={'class': 'searchhitinfobar'}) + \
                  f.text('%s - %s %s' % (size_str, rev_str, lastmod_str)) + \
@@ -753,7 +753,7 @@ class SearchResults:
 
     def querystring(self, querydict=None):
         """ Return query string, used in the page link
-        
+
         @keyword querydict: use these parameters (default: None)
         """
         if querydict is None:
@@ -767,7 +767,7 @@ class SearchResults:
 
     def formatInfo(self, formatter, page):
         """ Return formatted match info
-        
+
         @param formatter: the formatter instance to use
         @param page: the current page instance
         """
@@ -805,7 +805,7 @@ class SearchResults:
         self.request = request
         # Use 1 match, 2 matches...
         _ = request.getText
-        self.matchLabel = (_('match', formatted=False), _('matches', formatted=False))
+        self.matchLabel = (_('match'), _('matches'))
 
 
 def getSearchResults(request, query, hits, start, sort, estimated_hits):
