@@ -25,7 +25,7 @@ class Theme(ThemeBase):
 
     def footer(self, d, **keywords):
         """ Assemble wiki footer
-        
+
         @param d: parameter dictionary
         @keyword ...:...
         @rtype: unicode
@@ -60,7 +60,7 @@ class Theme(ThemeBase):
     def iconbar(self, d):
         """
         Assemble the iconbar
-        
+
         @param d: parameter dictionary
         @rtype: string
         @return: iconbar html
@@ -73,7 +73,7 @@ class Theme(ThemeBase):
                 if icon == "up":
                     if d['page_parent_page']:
                         iconbar.append('<li>%s</li>\n' % self.make_iconlink(icon, d))
-                elif icon == "subscribe" and self.cfg.mail_enabled:
+                elif icon == "subscribe" and (self.cfg.mail_enabled or self.cfg.jabber_enabled):
                     iconbar.append('<li>%s</li>\n' % self.make_iconlink(
                         ["subscribe", "unsubscribe"][self.request.user.isSubscribedTo([d['page_name']])], d))
                 else:
@@ -84,7 +84,7 @@ class Theme(ThemeBase):
     def header(self, d):
         """
         Assemble page header
-        
+
         @param d: parameter dictionary
         @rtype: string
         @return: page header html
@@ -131,7 +131,7 @@ class Theme(ThemeBase):
     def editorheader(self, d):
         """
         Assemble page header for editor
-        
+
         @param d: parameter dictionary
         @rtype: string
         @return: page header html
@@ -162,7 +162,7 @@ class Theme(ThemeBase):
     def edit_link(self, d, **keywords):
         """
         Assemble EditText link (or indication that page cannot be edited)
-        
+
         @param d: parameter dictionary
         @rtype: string
         @return: edittext link html
@@ -173,7 +173,7 @@ class Theme(ThemeBase):
     def availableactions(self, d):
         """
         assemble HTML code for the available actions
-        
+
         @param d: parameter dictionary
         @rtype: string
         @return: available actions html
@@ -188,26 +188,26 @@ class Theme(ThemeBase):
             available = list(available)
             available.sort()
             for action in available:
-                # Always add spaces: AttachFile -> Attach File 
+                # Always add spaces: AttachFile -> Attach File
                 # XXX do not make a page object just for split_title
                 #title = Page(request, action).split_title(force=1)
                 title = action
                 # Use translated version if available
-                title = _(title, formatted=False)
+                title = _(title)
                 querystr = {'action': action}
                 if rev:
                     querystr['rev'] = str(rev)
                 link = page.link_to(request, text=title, querystr=querystr, rel='nofollow')
                 html.append(link)
 
-        title = _("DeleteCache", formatted=False)
+        title = _("DeleteCache")
         link = page.link_to(request, text=title, querystr={'action': 'refresh'}, rel='nofollow')
 
         cache = caching.CacheEntry(request, page, page.getFormatterName(), scope='item')
         date = request.user.getFormattedDateTime(cache.mtime())
-        deletecache = u'<p>%s %s</p>' % (link, _('(cached %s)', formatted=False) % date)
+        deletecache = u'<p>%s %s</p>' % (link, _('(cached %s)') % date)
 
-        html = deletecache + u'<p>%s %s</p>\n' % (_('Or try one of these actions:', formatted=False),
+        html = deletecache + u'<p>%s %s</p>\n' % (_('Or try one of these actions:'),
                                        u', '.join(html))
         return html
 
@@ -215,7 +215,7 @@ class Theme(ThemeBase):
 def execute(request):
     """
     Generate and return a theme object
-        
+
     @param request: the request object
     @rtype: MoinTheme
     @return: Theme object
