@@ -1,12 +1,12 @@
 # -*- coding: iso-8859-1 -*-
 """
     MoinMoin - User Accounts
-    
+
     This module contains functions to access user accounts (list all users, get
     some specific user). User instances are used to access the user profile of
     some specific user (name, password, email, bookmark, trail, settings, ...).
 
-    The UserPreferences form support stuff is in module userform.
+    Some related code is in the userform and userprefs modules.
 
     TODO:
     * code is a mixture of highlevel user stuff and lowlevel storage functions,
@@ -20,12 +20,15 @@
 """
 
 # add names here to hide them in the cgitb traceback
-unsafe_names = ("id", "key", "val", "user_data", "enc_password")
+unsafe_names = ("id", "key", "val", "user_data", "enc_password", "recoverpass_key")
 
-import os, time, sha, codecs
+import os, time, codecs, base64
 
-from MoinMoin import config, caching, wikiutil, i18n
-from MoinMoin.util import timefuncs, filesys
+from MoinMoin.support.python_compatibility import hash_new, hmac_new
+
+from MoinMoin import config, caching, wikiutil, i18n, events
+from MoinMoin.util import timefuncs, filesys, random_string
+from MoinMoin.wikiutil import url_quote_plus
 
 
 def getUserList(request):
@@ -41,13 +44,68 @@ def getUserList(request):
     userlist = [f for f in files if user_re.match(f)]
     return userlist
 
-def get_by_email_address(request, email_address):
-    """ Searches for a user with a particular e-mail address and returns it. """
+def get_by_filter(request, filter_func):
+    """ Searches for an user with a given filter function """
     for uid in getUserList(request):
         theuser = User(request, uid)
-        if theuser.valid and theuser.email.lower() == email_address.lower():
+        if filter_func(theuser):
             return theuser
 
+def get_by_email_address(request, email_address):
+    """ Searches for an user with a particular e-mail address and returns it. """
+    filter_func = lambda user: user.valid and user.email.lower() == email_address.lower()
+    return get_by_filter(request, filter_func)
+
+def get_by_jabber_id(request, jabber_id):
+    """ Searches for an user with a perticular jabber id and returns it. """
+    filter_func = lambda user: user.valid and user.jid.lower() == jabber_id.lower()
+    return get_by_filter(request, filter_func)
+
+def _getUserIdByKey(request, key, search):
+    """ Get the user ID for a specified key/value pair.
+
+    This method must only be called for keys that are
+    guaranteed to be unique.
+
+    @param key: the key to look in
+    @param search: the value to look for
+    @return the corresponding user ID or None
+    """
+    if not search or not key:
+        return None
+    cfg = request.cfg
+    cachekey = '%s2id' % key
+    try:
+        _key2id = getattr(cfg.cache, cachekey)
+    except AttributeError:
+        arena = 'user'
+        cache = caching.CacheEntry(request, arena, cachekey, scope='wiki', use_pickle=True)
+        try:
+            _key2id = cache.content()
+        except caching.CacheError:
+            _key2id = {}
+        setattr(cfg.cache, cachekey, _key2id)
+    uid = _key2id.get(search, None)
+    if uid is None:
+        for userid in getUserList(request):
+            u = User(request, id=userid)
+            if hasattr(u, key):
+                value = getattr(u, key)
+                if isinstance(value, list):
+                    for val in value:
+                        _key2id[val] = userid
+                else:
+                    _key2id[value] = userid
+        arena = 'user'
+        cache = caching.CacheEntry(request, arena, cachekey, scope='wiki', use_pickle=True)
+        try:
+            cache.update(_key2id)
+        except caching.CacheError:
+            pass
+        uid = _key2id.get(search, None)
+    return uid
+
+
 def getUserId(request, searchName):
     """ Get the user ID for a specific user NAME.
 
@@ -55,40 +113,22 @@ def getUserId(request, searchName):
     @rtype: string
     @return: the corresponding user ID or None
     """
-    if not searchName:
-        return None
-    cfg = request.cfg
-    try:
-        _name2id = cfg.cache.name2id
-    except AttributeError:
-        arena = 'user'
-        key = 'name2id'
-        cache = caching.CacheEntry(request, arena, key, scope='wiki', use_pickle=True)
-        try:
-            _name2id = cache.content()
-        except caching.CacheError:
-            _name2id = {}
-        cfg.cache.name2id = _name2id
-    uid = _name2id.get(searchName, None)
-    if uid is None:
-        for userid in getUserList(request):
-            name = User(request, id=userid).name
-            _name2id[name] = userid
-        cfg.cache.name2id = _name2id
-        arena = 'user'
-        key = 'name2id'
-        cache = caching.CacheEntry(request, arena, key, scope='wiki', use_pickle=True)
-        try:
-            cache.update(_name2id)
-        except caching.CacheError:
-            pass
-        uid = _name2id.get(searchName, None)
-    return uid
+    return _getUserIdByKey(request, 'name', searchName)
+
+
+def getUserIdByOpenId(request, openid):
+    """ Get the user ID for a specific OpenID.
+
+    @param openid: the openid to look up
+    @rtype: string
+    @return: the corresponding user ID or None
+    """
+    return _getUserIdByKey(request, 'openids', openid)
 
 
 def getUserIdentification(request, username=None):
     """ Return user name or IP or '<unknown>' indicator.
-    
+
     @param request: the request object
     @param username: (optional) user name
     @rtype: string
@@ -99,33 +139,27 @@ def getUserIdentification(request, username=None):
     if username is None:
         username = request.user.name
 
-    return username or (request.cfg.show_hosts and request.remote_addr) or _("<unknown>", formatted=False)
+    return username or (request.cfg.show_hosts and request.remote_addr) or _("<unknown>")
 
 
-def encodePassword(pwd, charset='utf-8'):
+def encodePassword(pwd, salt=None):
     """ Encode a cleartext password
 
-    Compatible to Apache htpasswd SHA encoding.
-
-    When using different encoding than 'utf-8', the encoding might fail
-    and raise UnicodeError.
-
     @param pwd: the cleartext password, (unicode)
-    @param charset: charset used to encode password, used only for
-        compatibility with old passwords generated on moin-1.2.
+    @param salt: the salt for the password (string)
     @rtype: string
     @return: the password in apache htpasswd compatible SHA-encoding,
         or None
     """
-    import base64
+    pwd = pwd.encode('utf-8')
 
-    # Might raise UnicodeError, but we can't do anything about it here,
-    # so let the caller handle it.
-    pwd = pwd.encode(charset)
+    if salt is None:
+        salt = random_string(20)
+    assert isinstance(salt, str)
+    hash = hash_new('sha1', pwd)
+    hash.update(salt)
 
-    pwd = sha.new(pwd).digest()
-    pwd = '{SHA}' + base64.encodestring(pwd).rstrip()
-    return pwd
+    return '{SSHA}' + base64.encodestring(hash.digest() + salt).rstrip()
 
 
 def normalizeName(name):
@@ -139,7 +173,7 @@ def normalizeName(name):
     in user names.
 
     Prevent using ':' and ',' which are reserved by acl.
-    
+
     @param name: user name, unicode
     @rtype: unicode
     @return: user name that can be used in acl lines
@@ -150,7 +184,7 @@ def normalizeName(name):
     # Strip non alpha numeric characters (except username_allowedchars), keep white space
     name = ''.join([c for c in name if c.isalnum() or c.isspace() or c in username_allowedchars])
 
-    # Normalize white space. Each name can contain multiple 
+    # Normalize white space. Each name can contain multiple
     # words separated with only one space.
     name = ' '.join(name.split())
 
@@ -170,7 +204,7 @@ def encodeList(items):
     """ Encode list of items in user data file
 
     Items are separated by '\t' characters.
-    
+
     @param items: list unicode strings
     @rtype: unicode
     @return: list encoded as unicode
@@ -187,7 +221,7 @@ def encodeList(items):
 
 def decodeList(line):
     """ Decode list of items from user data file
-    
+
     @param line: line containing list of items, encoded with encodeList
     @rtype: list of unicode strings
     @return: list of items in encoded in line
@@ -205,7 +239,7 @@ def encodeDict(items):
 
     Items are separated by '\t' characters.
     Each item is key:value.
-    
+
     @param items: dict of unicode:unicode
     @rtype: unicode
     @return: dict encoded as unicode
@@ -219,7 +253,7 @@ def encodeDict(items):
 
 def decodeDict(line):
     """ Decode dict of key:value pairs from user data file
-    
+
     @param line: line containing a dict, encoded with encodeDict
     @rtype: dict
     @return: dict  unicode:unicode items
@@ -228,7 +262,7 @@ def decodeDict(line):
     for item in line.split('\t'):
         item = item.strip()
         if not item:
-            continue        
+            continue
         key, value = item.split(':', 1)
         items[key] = value
     return items
@@ -239,7 +273,7 @@ class User:
 
     def __init__(self, request, id=None, name="", password=None, auth_username="", **kw):
         """ Initialize User object
-        
+
         TODO: when this gets refactored, use "uid" not builtin "id"
 
         @param request: the request object
@@ -252,12 +286,11 @@ class User:
                               default: 'internal'
         @keyword auth_attribs: tuple of user object attribute names that are
                                determined by auth method and should not be
-                               changed by UserPreferences form, default: ().
+                               changeable by preferences, default: ().
                                First tuple element was used for authentication.
         """
         self._cfg = request.cfg
         self.valid = 0
-        self.trusted = 0
         self.id = id
         self.auth_username = auth_username
         self.auth_method = kw.get('auth_method', 'internal')
@@ -269,31 +302,29 @@ class User:
 
         if name:
             self.name = name
-        elif auth_username: # this is needed for user_autocreate
+        elif auth_username: # this is needed for user autocreate
             self.name = auth_username
 
         # create checkbox fields (with default 0)
         for key, label in self._cfg.user_checkbox_fields:
             setattr(self, key, self._cfg.user_checkbox_defaults.get(key, 0))
 
-        self.enc_password = ""
+        self.recoverpass_key = ""
+
         if password:
-            if password.startswith('{SHA}'):
-                self.enc_password = password
-            else:
-                try:
-                    self.enc_password = encodePassword(password)
-                except UnicodeError:
-                    pass # Should never happen
+            self.enc_password = encodePassword(password)
 
         #self.edit_cols = 80
         self.tz_offset = int(float(self._cfg.tz_offset) * 3600)
         self.language = ""
         self.real_language = "" # In case user uses "Browser setting". For language-statistics
+        self._stored = False
         self.date_fmt = ""
         self.datetime_fmt = ""
         self.quicklinks = self._cfg.quicklinks_default
         self.subscribed_pages = self._cfg.subscribed_pages_default
+        self.email_subscribed_events = self._cfg.email_subscribed_events_default
+        self.jabber_subscribed_events = self._cfg.jabber_subscribed_events_default
         self.theme_name = self._cfg.theme_default
         self.editor_default = self._cfg.editor_default
         self.editor_ui = self._cfg.editor_ui
@@ -304,23 +335,23 @@ class User:
         self._trail = []
 
         # we got an already authenticated username:
-        check_pass = 0
+        check_password = None
         if not self.id and self.auth_username:
             self.id = getUserId(request, self.auth_username)
             if not password is None:
-                check_pass = 1
+                check_password = password
         if self.id:
-            self.load_from_id(check_pass)
-            if self.name == self.auth_username:
-                self.trusted = 1
+            self.load_from_id(check_password)
         elif self.name:
             self.id = getUserId(self._request, self.name)
             if self.id:
-                self.load_from_id(1)
-            else:
-                self.id = self.make_id()
-        else:
+                # no password given should fail
+                self.load_from_id(password or u'')
+        # Still no ID - make new user
+        if not self.id:
             self.id = self.make_id()
+            if password is not None:
+                self.enc_password = encodePassword(password)
 
         # "may" so we can say "if user.may.read(pagename):"
         if self._cfg.SecurityPolicy:
@@ -350,13 +381,12 @@ class User:
 
         @param changed: bool, set this to True if you updated the user profile values
         """
-        if self._cfg.user_autocreate:
-            if not self.valid and not self.disabled or changed: # do we need to save/update?
-                self.save() # yes, create/update user profile
+        if not self.valid and not self.disabled or changed: # do we need to save/update?
+            self.save() # yes, create/update user profile
 
     def __filename(self):
         """ Get filename of the user's file on disk
-        
+
         @rtype: string
         @return: full path and filename of user account file
         """
@@ -364,22 +394,22 @@ class User:
 
     def exists(self):
         """ Do we have a user account for this user?
-        
+
         @rtype: bool
         @return: true, if we have a user account
         """
         return os.path.exists(self.__filename())
 
-    def load_from_id(self, check_pass=0):
+    def load_from_id(self, password=None):
         """ Load user account data from disk.
 
         Can only load user data if the id number is already known.
 
         This loads all member variables, except "id" and "valid" and
         those starting with an underscore.
-        
-        @param check_pass: If 1, then self.enc_password must match the
-                           password in the user account file.
+
+        @param password: If not None, then the given password must match the
+                         password in the user account file.
         """
         if not self.exists():
             return
@@ -403,7 +433,7 @@ class User:
                         val = decodeDict(val)
                     # for compatibility reading old files, keep these explicit
                     # we will store them with [] appended
-                    elif key in ['quicklinks', 'subscribed_pages']:
+                    elif key in ['quicklinks', 'subscribed_pages', 'subscribed_events']:
                         val = decodeList(val)
                     user_data[key] = val
             except ValueError:
@@ -413,16 +443,11 @@ class User:
         # values, we set 'changed' flag, and later save the user data.
         changed = 0
 
-        if check_pass:
-            # If we have no password set, we don't accept login with username
-            if not user_data['enc_password']:
-                return
-            # Check for a valid password, possibly changing encoding
-            valid, changed = self._validatePassword(user_data)
+        if password is not None:
+            # Check for a valid password, possibly changing storage
+            valid, changed = self._validatePassword(user_data, password)
             if not valid:
                 return
-            else:
-                self.trusted = 1
 
         # Remove ignored checkbox values from user data
         for key, label in self._cfg.user_checkbox_fields:
@@ -459,79 +484,50 @@ class User:
         if not self.disabled:
             self.valid = 1
 
+        # Mark this user as stored so saves don't send
+        # the "user created" event
+        self._stored = True
+
         # If user data has been changed, save fixed user data.
         if changed:
             self.save()
 
-    def _validatePassword(self, data):
-        """ Try to validate user password
+    def _validatePassword(self, data, password):
+        """
+        Check user password.
 
         This is a private method and should not be used by clients.
 
-        In pre 1.3, the wiki used some 8 bit charset. The user password
-        was entered in this 8 bit password and passed to
-        encodePassword. So old passwords can use any of the charset
-        used.
-
-        In 1.3, we use unicode internally, so we encode the password in
-        encodePassword using utf-8.
-
-        When we compare passwords we must compare with same encoding, or
-        the passwords will not match. We don't know what encoding the
-        password on the user file uses. We may ask the wiki admin to put
-        this into the config, but he may be wrong.
-
-        The way chosen is to try to encode and compare passwords using
-        all the encoding that were available on 1.2, until we get a
-        match, which means that the user is valid.
-
-        If we get a match, we replace the user password hash with the
-        utf-8 encoded version, and next time it will match on first try
-        as before. The user password did not change, this change is
-        completely transparent for the user. Only the sha digest will
-        change.
-
-        @param data: dict with user data
+        @param data: dict with user data (from storage)
+        @param password: password to verify
         @rtype: 2 tuple (bool, bool)
-        @return: password is valid, password did change
+        @return: password is valid, enc_password changed
         """
-        # First try with default encoded password. Match only non empty
-        # passwords. (require non empty enc_password)
-        if self.enc_password and self.enc_password == data['enc_password']:
-            return True, False
+        epwd = data['enc_password']
 
-        # Try to match using one of pre 1.3 8 bit charsets
+        # If we have no password set, we don't accept login with username
+        if not epwd:
+            return False, False
 
-        # Get the clear text password from the form (require non empty
-        # password)
-        password = self._request.form.get('password', [None])[0]
+        # require non empty password
         if not password:
             return False, False
 
-        # First get all available pre13 charsets on this system
-        pre13 = ['iso-8859-1', 'iso-8859-2', 'euc-jp', 'gb2312', 'big5', ]
-        available = []
-        for charset in pre13:
-            try:
-                encoder = codecs.getencoder(charset)
-                available.append(charset)
-            except LookupError:
-                pass # missing on this system
+        password = password.encode('utf-8')
 
-        # Now try to match the password
-        for charset in available:
-            # Try to encode, failure is expected
-            try:
-                enc_password = encodePassword(password, charset=charset)
-            except UnicodeError:
-                continue
-
-            # And match (require non empty enc_password)
-            if enc_password and enc_password == data['enc_password']:
-                # User password match - replace the user password in the
-                # file with self.password
-                data['enc_password'] = self.enc_password
+        if epwd[:5] == '{SHA}':
+            enc = '{SHA}' + base64.encodestring(hash_new('sha1', password).digest()).rstrip()
+            if epwd == enc:
+                data['enc_password'] = encodePassword(password)
                 return True, True
+            return False, False
+
+        if epwd[:6] == '{SSHA}':
+            data = base64.decodestring(epwd[6:])
+            salt = data[20:]
+            hash = hash_new('sha1', password)
+            hash.update(salt)
+            return hash.digest() == data[:20], False
 
         # No encoded password match, this must be wrong password
         return False, False
@@ -580,21 +576,32 @@ class User:
 
         arena = 'user'
         key = 'name2id'
-        cache = caching.CacheEntry(self._request, arena, key, scope='wiki').remove()
+        caching.CacheEntry(self._request, arena, key, scope='wiki').remove()
         try:
             del self._request.cfg.cache.name2id
         except:
             pass
-        
+        key = 'openid2id'
+        caching.CacheEntry(self._request, arena, key, scope='wiki').remove()
+        try:
+            del self._request.cfg.cache.openid2id
+        except:
+            pass
+
         if not self.disabled:
             self.valid = 1
 
+        if not self._stored:
+            self._stored = True
+            event = events.UserCreatedEvent(self._request, self)
+            events.send_event(event)
+
     # -----------------------------------------------------------------
     # Time and date formatting
 
     def getTime(self, tm):
         """ Get time in user's timezone.
-        
+
         @param tm: time (UTC UNIX timestamp)
         @rtype: int
         @return: tm tuple adjusted for user's timezone
@@ -628,7 +635,7 @@ class User:
 
     def setBookmark(self, tm):
         """ Set bookmark timestamp.
-        
+
         @param tm: timestamp
         """
         if self.valid:
@@ -639,7 +646,7 @@ class User:
 
     def getBookmark(self):
         """ Get bookmark timestamp.
-        
+
         @rtype: int
         @return: bookmark timestamp or None
         """
@@ -673,7 +680,7 @@ class User:
 
     def getSubscriptionList(self):
         """ Get list of pages this user has subscribed to
-        
+
         @rtype: list
         @return: pages this user has subscribed to
         """
@@ -681,13 +688,13 @@ class User:
 
     def isSubscribedTo(self, pagelist):
         """ Check if user subscription matches any page in pagelist.
-        
+
         The subscription list may contain page names or interwiki page
         names. e.g 'Page Name' or 'WikiName:Page_Name'
-        
-        TODO: check if its fast enough when calling with many users
-        from page.getSubscribersList()
-        
+
+        TODO: check if it's fast enough when getting called for many
+              users from page.getSubscribersList()
+
         @param pagelist: list of pages to check for subscription
         @rtype: bool
         @return: if user is subscribed any page in pagelist
@@ -734,6 +741,11 @@ class User:
         if pagename not in self.subscribed_pages:
             self.subscribed_pages.append(pagename)
             self.save()
+
+            # Send a notification
+            from MoinMoin.events import SubscribedToPageEvent, send_event
+            e = SubscribedToPageEvent(self._request, pagename, self.name)
+            send_event(e)
             return True
 
         return False
@@ -784,7 +796,7 @@ class User:
 
     def isQuickLinkedTo(self, pagelist):
         """ Check if user quicklink matches any page in pagelist.
-        
+
         @param pagelist: list of pages to check for quicklinks
         @rtype: bool
         @return: if user has quicklinked any page in pagelist
@@ -802,8 +814,8 @@ class User:
         return False
 
     def addQuicklink(self, pagename):
-        """ Adds a page to the user quicklinks 
-        
+        """ Adds a page to the user quicklinks
+
         If the wiki has an interwiki name, all links are saved as
         interwiki names. If not, as simple page name.
 
@@ -831,10 +843,10 @@ class User:
         return changed
 
     def removeQuicklink(self, pagename):
-        """ Remove a page from user quicklinks 
-        
+        """ Remove a page from user quicklinks
+
         Remove both interwiki and simple name from quicklinks.
-        
+
         @param pagename: page name
         @type pagename: unicode
         @rtype: bool
@@ -854,7 +866,7 @@ class User:
         return changed
 
     def _interWikiName(self, pagename):
-        """ Return the inter wiki name of a page name 
+        """ Return the inter wiki name of a page name
 
         @param pagename: page name
         @type pagename: unicode
@@ -868,12 +880,12 @@ class User:
     # Trail
 
     def _wantTrail(self):
-        return (not self.valid and self._request.session != {}  # anon session
+        return (not self.valid and self._request.session.is_stored  # anon session
                 or self.valid and (self.show_page_trail or self.remember_last_visit))  # logged-in session
 
     def addTrail(self, page):
         """ Add page to trail.
-        
+
         @param page: the page (object) to add to the trail
         """
         if self._wantTrail():
@@ -900,12 +912,13 @@ class User:
             self.saveTrail()
 
     def saveTrail(self):
-        """Save trail in session"""
-        self._request.session['trail'] = self._trail
+        """ Save trail into session """
+        if not self._request.session.is_new:
+            self._request.session['trail'] = self._trail
 
     def getTrail(self):
         """ Return list of recently visited pages.
-        
+
         @rtype: list
         @return: pages in trail
         """
@@ -927,7 +940,7 @@ class User:
     def isSuperUser(self):
         """ Check if this user is superuser """
         request = self._request
-        if request.cfg.DesktopEdition and request.remote_addr == '127.0.0.1' and request.user.valid:
+        if request.cfg.DesktopEdition and request.remote_addr == '127.0.0.1' and request.user and request.user.valid:
             # the DesktopEdition gives any local user superuser powers
             return True
         superusers = request.cfg.superuser
@@ -938,7 +951,7 @@ class User:
         """ Return user host """
         _ = self._request.getText
         host = self.isCurrentUser() and self._cfg.show_hosts and self._request.remote_addr
-        return host or _("<unknown>", formatted=False)
+        return host or _("<unknown>")
 
     def wikiHomeLink(self):
         """ Return wiki markup usable as a link to the user homepage,
@@ -956,10 +969,10 @@ class User:
 
     def signature(self):
         """ Return user signature using wiki markup
-        
+
         Users sign with a link to their homepage.
         Visitors return their host address.
-        
+
         TODO: The signature use wiki format only, for example, it will
         not create a link when using rst format. It will also break if
         we change wiki syntax.
@@ -969,6 +982,35 @@ class User:
         else:
             return self.host()
 
+    def generate_recovery_token(self):
+        key = random_string(64, "abcdefghijklmnopqrstuvwxyz0123456789")
+        msg = str(int(time.time()))
+        h = hmac_new(key, msg).hexdigest()
+        self.recoverpass_key = key
+        self.save()
+        return msg + '-' + h
+
+    def apply_recovery_token(self, tok, newpass):
+        key = self.recoverpass_key
+        parts = tok.split('-')
+        if len(parts) != 2:
+            return False
+        try:
+            stamp = int(parts[0])
+        except ValueError:
+            return False
+        # only allow it to be valid for twelve hours
+        if stamp + 12*60*60 < time.time():
+            return False
+        # check hmac
+        h = hmac_new(self.recoverpass_key, str(stamp)).hexdigest()
+        if h != parts[1]:
+            return False
+        self.recoverpass_key = ""
+        self.enc_password = encodePassword(newpass)
+        self.save()
+        return True
+
     def mailAccountData(self, cleartext_passwd=None):
         """ Mail a user who forgot his password a message enabling
             him to login again.
@@ -977,42 +1019,32 @@ class User:
         from MoinMoin.wikiutil import getLocalizedPage
         _ = self._request.getText
 
-        if not self.enc_password: # generate pw if there is none yet
-            from random import randint
-            import base64
-
-            charset = 'utf-8'
-            pwd = "%s%d" % (str(time.time()), randint(0, 65535))
-            pwd = pwd.encode(charset)
-
-            pwd = sha.new(pwd).digest()
-            pwd = '{SHA}%s' % base64.encodestring(pwd).rstrip()
-
-            self.enc_password = pwd
-            self.save()
+        tok = self.generate_recovery_token()
 
         text = '\n' + _("""\
 Login Name: %s
 
-Login Password: %s
+Password recovery token: %s
 
-Login URL: %s/%s?action=login
-""", formatted=False) % (
-                        self.name, self.enc_password, self._request.getBaseURL(), getLocalizedPage(self._request, 'UserPreferences').page_name)
+Password reset URL: %s/?action=recoverpass&name=%s&token=%s
+""") % (
+                        self.name,
+                        tok,
+                        self._request.getBaseURL(),
+                        url_quote_plus(self.name),
+                        tok, )
 
         text = _("""\
-Somebody has requested to submit your account data to this email address.
+Somebody has requested to email you a password recovery token.
 
-If you lost your password, please use the data below and just enter the
-password AS SHOWN into the wiki's password form field (use copy and paste
-for that).
-
-After successfully logging in, it is of course a good idea to set a new and known password.
-""", formatted=False) + text
+If you lost your password, please go to the password reset URL below or
+go to the password recovery page again and enter your username and the
+recovery token.
+""") + text
 
 
         subject = _('[%(sitename)s] Your wiki account data',
-                    formatted=False) % {'sitename': self._cfg.sitename or "Wiki"}
+                ) % {'sitename': self._cfg.sitename or "Wiki"}
         mailok, msg = sendmail.sendmail(self._request, [self.email], subject,
                                     text, mail_from=self._cfg.mail_from)
         return mailok, msg
