@@ -5,7 +5,7 @@
     @copyright: 2000-2004 Juergen Hermann <jh@web.de>,
                 2004 by Florian Festi,
                 2006 by Mikko Virkkil,
-                2005-2007 MoinMoin:ThomasWaldmann,
+                2005-2008 MoinMoin:ThomasWaldmann,
                 2007 MoinMoin:ReimarBauer
     @license: GNU GPL, see COPYING for details.
 """
@@ -16,7 +16,9 @@ import os
 import re
 import time
 import urllib
-import logging
+
+from MoinMoin import log
+logging = log.getLogger(__name__)
 
 from MoinMoin import config
 from MoinMoin.util import pysupport, lock
@@ -247,6 +249,7 @@ def escape(s, quote=0):
     s = s.replace(">", "&gt;")
     if quote:
         s = s.replace('"', "&quot;")
+        s = s.replace("'", "&#x27;")
     return s
 
 def clean_input(text, max_len=201):
@@ -549,7 +552,7 @@ def load_wikimap(request):
         lines = []
 
         for filename in request.cfg.shared_intermap_files:
-            f = open(filename, "r")
+            f = codecs.open(filename, "r", config.charset)
             lines.extend(f.readlines())
             f.close()
 
@@ -703,7 +706,7 @@ def isTemplatePage(request, pagename):
     @rtype: bool
     @return: true if page is a template page
     """
-    return request.cfg.cache.page_template_regex.search(pagename) is not None
+    return request.cfg.cache.page_template_regexact.search(pagename) is not None
 
 
 def isGroupPage(request, pagename):
@@ -713,7 +716,7 @@ def isGroupPage(request, pagename):
     @rtype: bool
     @return: true if page is a form page
     """
-    return request.cfg.cache.page_group_regex.search(pagename) is not None
+    return request.cfg.cache.page_group_regexact.search(pagename) is not None
 
 
 def filterCategoryPages(request, pagelist):
@@ -730,7 +733,7 @@ def filterCategoryPages(request, pagelist):
     @rtype: list
     @return: only the category pages of pagelist
     """
-    func = request.cfg.cache.page_category_regex.search
+    func = request.cfg.cache.page_category_regexact.search
     return [pn for pn in pagelist if func(pn)]
 
 
@@ -749,7 +752,7 @@ def getLocalizedPage(request, pagename): # was: getSysPage
              if it exists
     """
     from MoinMoin.Page import Page
-    i18n_name = request.getText(pagename, formatted=False)
+    i18n_name = request.getText(pagename)
     pageobj = None
     if i18n_name != pagename:
         if request.page and i18n_name == request.page.page_name:
@@ -929,6 +932,9 @@ MIMETYPES_MORE = {
  '.patch': 'text/x-diff',
  '.diff': 'text/x-diff',
  '.py': 'text/x-python',
+ '.cfg': 'text/plain',
+ '.conf': 'text/plain',
+ '.irc': 'text/plain',
 }
 [mimetypes.add_type(mimetype, ext, True) for ext, mimetype in MIMETYPES_MORE.items()]
 
@@ -995,9 +1001,7 @@ class MimeType(object):
             type "wiki" instead of "text/moin-wiki".
         """
         format = format.lower()
-        if format in ('plain', 'csv', 'rst', 'docbook', 'latex', 'tex', 'html', 'css',
-                      'xml', 'python', 'perl', 'php', 'ruby', 'javascript',
-                      'cplusplus', 'java', 'pascal', 'diff', 'gettext', 'xslt', 'creole', ):
+        if format in config.parser_text_mimetype:
             mimetype = 'text', format
         else:
             mapping = {
@@ -1079,10 +1083,12 @@ class PluginAttributeError(PluginError):
 
 def importPlugin(cfg, kind, name, function="execute"):
     """ Import wiki or builtin plugin
-    
-    Returns function from a plugin module name. If name can not be
-    imported, raise PluginMissingError. If function is missing, raise
-    PluginAttributeError.
+
+    Returns <function> attr from a plugin module <name>.
+    If <function> attr is missing, raise PluginAttributeError.
+    If <function> is None, return the whole module object.
+
+    If <name> plugin can not be imported, raise PluginMissingError.
 
     kind may be one of 'action', 'formatter', 'macro', 'parser' or any other
     directory that exist in MoinMoin or data/plugin.
@@ -1090,7 +1096,7 @@ def importPlugin(cfg, kind, name, function="execute"):
     Wiki plugins will always override builtin plugins. If you want
     specific plugin, use either importWikiPlugin or importBuiltinPlugin
     directly.
-    
+
     @param cfg: wiki config instance
     @param kind: what kind of module we want to import
     @param name: the name of the module
@@ -1106,41 +1112,56 @@ def importPlugin(cfg, kind, name, function="execute"):
 
 def importWikiPlugin(cfg, kind, name, function="execute"):
     """ Import plugin from the wiki data directory
-    
+
     See importPlugin docstring.
     """
-    if not name in wikiPlugins(kind, cfg):
-        raise PluginMissingError
-    moduleName = '%s.plugin.%s.%s' % (cfg.siteid, kind, name)
+    plugins = wikiPlugins(kind, cfg)
+    modname = plugins.get(name, None)
+    if modname is None:
+        raise PluginMissingError()
+    moduleName = '%s.%s' % (modname, name)
     return importNameFromPlugin(moduleName, function)
 
 
 def importBuiltinPlugin(kind, name, function="execute"):
-    """ Import builtin plugin from MoinMoin package 
-    
+    """ Import builtin plugin from MoinMoin package
+
     See importPlugin docstring.
     """
     if not name in builtinPlugins(kind):
-        raise PluginMissingError
+        raise PluginMissingError()
     moduleName = 'MoinMoin.%s.%s' % (kind, name)
     return importNameFromPlugin(moduleName, function)
 
 
 def importNameFromPlugin(moduleName, name):
-    """ Return name from plugin module 
-    
-    Raise PluginAttributeError if name does not exists.
+    """ Return <name> attr from <moduleName> module,
+        raise PluginAttributeError if name does not exist.
+
+        If name is None, return the <moduleName> module object.
     """
-    module = __import__(moduleName, globals(), {}, [name])
-    try:
-        return getattr(module, name)
-    except AttributeError:
-        raise PluginAttributeError
+    if name is None:
+        fromlist = []
+    else:
+        fromlist = [name]
+    module = __import__(moduleName, globals(), {}, fromlist)
+    if fromlist:
+        # module has the obj for module <moduleName>
+        try:
+            return getattr(module, name)
+        except AttributeError:
+            raise PluginAttributeError
+    else:
+        # module now has the toplevel module of <moduleName> (see __import__ docs!)
+        components = moduleName.split('.')
+        for comp in components[1:]:
+            module = getattr(module, comp)
+        return module
 
 
 def builtinPlugins(kind):
     """ Gets a list of modules in MoinMoin.'kind'
-    
+
     @param kind: what kind of modules we look for
     @rtype: list
     @return: module names
@@ -1150,32 +1171,38 @@ def builtinPlugins(kind):
 
 
 def wikiPlugins(kind, cfg):
-    """ Gets a list of modules in data/plugin/'kind'
- 
-    @param kind: what kind of modules we look for
-    @rtype: list
-    @return: module names
     """
-    # Wiki plugins are located in wikiconfig.plugin module
-    modulename = '%s.plugin.%s' % (cfg.siteid, kind)
+    Gets a dict containing the names of all plugins of @kind
+    as the key and the containing module name as the value.
 
-    # short-cut if we've loaded the list already
+    @param kind: what kind of modules we look for
+    @rtype: dict
+    @return: plugin name to containing module name mapping
+    """
+    # short-cut if we've loaded the dict already
     # (or already failed to load it)
-    if kind in cfg._site_plugin_lists:
-        return cfg._site_plugin_lists[kind]
-
-    try:
-        plugins = pysupport.importName(modulename, "modules")
-        cfg._site_plugin_lists[kind] = plugins
-        return plugins
-    except ImportError:
-        cfg._site_plugin_lists[kind] = []
-        return []
+    cache = cfg._site_plugin_lists
+    if kind in cache:
+        result = cache[kind]
+    else:
+        result = {}
+        for modname in cfg._plugin_modules:
+            try:
+                module = pysupport.importName(modname, kind)
+                packagepath = os.path.dirname(module.__file__)
+                plugins = pysupport.getPluginModules(packagepath)
+                for p in plugins:
+                    if not p in result:
+                        result[p] = '%s.%s' % (modname, kind)
+            except AttributeError:
+                pass
+        cache[kind] = result
+    return result
 
 
 def getPlugins(kind, cfg):
     """ Gets a list of plugin names of kind
-    
+
     @param kind: what kind of modules we look for
     @rtype: list
     @return: module names
@@ -1251,58 +1278,148 @@ def getParserForExtension(cfg, extension):
 ### Parameter parsing
 #############################################################################
 
-def parse_quoted_separated(args, separator=',', name_value=True, seplimit=0):
-    """
-    Parses the given arguments according to the other parameters.
-    If name_value is True, it parses keyword arguments (name=value)
-    and returns the keyword arguments in the second return value
-    (a dict) and positional arguments that occurred after any keyword
-    argument in the third return value (a list).
-    The first return value always contains the positional arguments,
-    if name_value is False only it is present.
+class BracketError(Exception):
+    pass
 
-    Arguments can be quoted with a double-quote ('"') and the quote
-    can be escaped by doubling it, the separator and equal sign (for
-    keyword args) can both be quoted, when keyword args are enabled
-    then the name of a keyword argument can also be quoted.
+class BracketUnexpectedCloseError(BracketError):
+    def __init__(self, bracket):
+        self.bracket = bracket
+        BracketError.__init__(self, "Unexpected closing bracket %s" % bracket)
+
+class BracketMissingCloseError(BracketError):
+    def __init__(self, bracket):
+        self.bracket = bracket
+        BracketError.__init__(self, "Missing closing bracket %s" % bracket)
+
+class ParserPrefix:
+    """
+    Trivial container-class holding a single character for
+    the possible prefixes for parse_quoted_separated_ext
+    and implementing rich equal comparison.
+    """
+    def __init__(self, prefix):
+        self.prefix = prefix
+
+    def __eq__(self, other):
+        return isinstance(other, ParserPrefix) and other.prefix == self.prefix
+
+    def __repr__(self):
+        return '<ParserPrefix(%s)>' % self.prefix.encode('utf-8')
+
+def parse_quoted_separated_ext(args, separator=None, name_value_separator=None,
+                               brackets=None, seplimit=0, multikey=False,
+                               prefixes=None, quotes='"'):
+    """
+    Parses the given string according to the other parameters.
+
+    Items can be quoted with any character from the quotes parameter
+    and each quote can be escaped by doubling it, the separator and
+    name_value_separator can both be quoted, when name_value_separator
+    is set then the name can also be quoted.
 
     Values that are not given are returned as None, while the
-    empty string as a value can be achieved by quoting it; keys
-    are never returned as None.
+    empty string as a value can be achieved by quoting it.
 
     If a name or value does not start with a quote, then the quote
-    character looses its special meaning for that name or value.
+    looses its special meaning for that name or value, unless it
+    starts with one of the given prefixes (the parameter is unicode
+    containing all allowed prefixes.) The prefixes will be returned
+    as ParserPrefix() instances in the first element of the tuple
+    for that particular argument.
+
+    If multiple separators follow each other, this is treated as
+    having None arguments inbetween, that is also true for when
+    space is used as separators (when separator is None), filter
+    them out afterwards.
+
+    The function can also do bracketing, i.e. parse expressions
+    that contain things like
+        "(a (a b))" to ['(', 'a', ['(', 'a', 'b']],
+    in this case, as in this example, the returned list will
+    contain sub-lists and the brackets parameter must be a list
+    of opening and closing brackets, e.g.
+        brackets = ['()', '<>']
+    Each sub-list's first item is the opening bracket used for
+    grouping.
+    Nesting will be observed between the different types of
+    brackets given. If bracketing doesn't match, a BracketError
+    instance is raised with a 'bracket' property indicating the
+    type of missing or unexpected bracket, the instance will be
+    either of the class BracketMissingCloseError or of the class
+    BracketUnexpectedCloseError.
+
+    If multikey is True (along with setting name_value_separator),
+    then the returned tuples for (key, value) pairs can also have
+    multiple keys, e.g.
+        "a=b=c" -> ('a', 'b', 'c')
 
     @param args: arguments to parse
-    @param separator: the argument separator, defaults to a comma (',')
-    @param name_value: indicates whether to parse keyword arguments
+    @param separator: the argument separator, defaults to None, meaning any
+        space separates arguments
+    @param name_value_separator: separator for name=value, default '=',
+        name=value keywords not parsed if evaluates to False
+    @param brackets: a list of two-character strings giving
+        opening and closing brackets
     @param seplimit: limits the number of parsed arguments
-    @rtype: tuple, list
-    @returns: if name_value is False, returns a list of arguments,
-              otherwise a list of positional, a dict of keyword and
-              a list of trailing arguments
+    @param multikey: multiple keys allowed for a single value
+    @rtype: list
+    @returns: list of unicode strings and tuples containing
+        unicode strings, or lists containing the same for
+        bracketing support
     """
     idx = 0
+    assert name_value_separator is None or name_value_separator != separator
+    assert name_value_separator is None or len(name_value_separator) == 1
     if not isinstance(args, unicode):
         raise TypeError('args must be unicode')
     max = len(args)
-    ret_positional = [] # positional argument return value
-    ret_trailing = []   # trailing arguments return value
-    positional = ret_positional
-    keyword = {}        # keyword arguments
-    curname = u''       # current name, initially value as well (name=value)
-    cur = None          # current value
-    cur_quoted = False  # indicates whether value was quoted,
-                        # needed None vs. u'' handling
-    quoted = False      # we're inside quotes
+    result = []         # result list
+    cur = [None]        # current item
+    quoted = None       # we're inside quotes, indicates quote character used
     skipquote = 0       # next quote is a quoted quote
     noquote = False     # no quotes expected because word didn't start with one
     seplimit_reached = False # number of separators exhausted
     separator_count = 0 # number of separators encountered
     SPACE = [' ', '\t', ]
     nextitemsep = [separator]   # used for skipping trailing space
-    if name_value:
-        nextitemsep.append('=')
+    SPACE = [' ', '\t', ]
+    if separator is None:
+        nextitemsep = SPACE[:]
+        separators = SPACE
+    else:
+        nextitemsep = [separator]   # used for skipping trailing space
+        separators = [separator]
+    if name_value_separator:
+        nextitemsep.append(name_value_separator)
+
+    # bracketing support
+    opening = []
+    closing = []
+    bracketstack = []
+    matchingbracket = {}
+    if brackets:
+        for o, c in brackets:
+            assert not o in opening
+            opening.append(o)
+            assert not c in closing
+            closing.append(c)
+            matchingbracket[o] = c
+
+    def additem(result, cur, separator_count, nextitemsep):
+        if len(cur) == 1:
+            result.extend(cur)
+        elif cur:
+            result.append(tuple(cur))
+        cur = [None]
+        noquote = False
+        separator_count += 1
+        seplimit_reached = False
+        if seplimit and separator_count >= seplimit:
+            seplimit_reached = True
+            nextitemsep = [n for n in nextitemsep if n in separators]
+
+        return cur, noquote, separator_count, seplimit_reached, nextitemsep
+
     while idx < max:
         char = args[idx]
         next = None
@@ -1310,7 +1427,7 @@ def parse_quoted_separated(args, separator=',', name_value=True, seplimit=0):
             next = args[idx+1]
         if skipquote:
             skipquote -= 1
-        if not quoted and char in SPACE:
+        if not separator is None and not quoted and char in SPACE:
             spaces = ''
             # accumulate all space
             while char in SPACE and idx < max - 1:
@@ -1324,70 +1441,110 @@ def parse_quoted_separated(args, separator=',', name_value=True, seplimit=0):
             if char in nextitemsep:
                 continue
             idx -= 1
-            if not cur is None:
-                if cur:
-                    cur = cur + spaces
-            elif curname:
-                curname = curname + spaces
-        elif not quoted and name_value and char == '=':
-            if cur is None:
-                cur = u''
-                cur_quoted = False
+            if len(cur) and cur[-1]:
+                cur[-1] = cur[-1] + spaces
+        elif not quoted and char == name_value_separator:
+            if multikey or len(cur) == 1:
+                cur.append(None)
             else:
-                cur += '='
+                if not multikey:
+                    if cur[-1] is None:
+                        cur[-1] = ''
+                    cur[-1] += name_value_separator
+                else:
+                    cur.append(None)
             noquote = False
-        elif not quoted and not seplimit_reached and char == separator:
-            if cur is None:
-                cur = curname
-                curname = None
-            if not cur and not cur_quoted:
-                cur = None
-            if curname is not None:
-                keyword[curname] = cur
-                positional = ret_trailing
-            else:
-                positional.append(cur)
-            curname = u''
-            cur = None
-            noquote = False
-            cur_quoted = False
-            separator_count += 1
-            if seplimit and separator_count >= seplimit:
-                seplimit_reached = True
-                nextitemsep.remove(separator)
-        elif not quoted and not noquote and char == '"':
-            quoted = True
-            cur_quoted = True
-        elif quoted and not skipquote and char == '"':
-            if next == '"':
+        elif not quoted and not seplimit_reached and char in separators:
+            (cur, noquote, separator_count, seplimit_reached,
+             nextitemsep) = additem(result, cur, separator_count, nextitemsep)
+        elif not quoted and not noquote and char in quotes:
+            if len(cur) and cur[-1] is None:
+                del cur[-1]
+            cur.append(u'')
+            quoted = char
+        elif char == quoted and not skipquote:
+            if next == quoted:
                 skipquote = 2 # will be decremented right away
             else:
-                quoted = False
+                quoted = None
+        elif not quoted and char in opening:
+            while len(cur) and cur[-1] is None:
+                del cur[-1]
+            (cur, noquote, separator_count, seplimit_reached,
+             nextitemsep) = additem(result, cur, separator_count, nextitemsep)
+            bracketstack.append((matchingbracket[char], result))
+            result = [char]
+        elif not quoted and char in closing:
+            while len(cur) and cur[-1] is None:
+                del cur[-1]
+            (cur, noquote, separator_count, seplimit_reached,
+             nextitemsep) = additem(result, cur, separator_count, nextitemsep)
+            cur = []
+            if not bracketstack:
+                raise BracketUnexpectedCloseError(char)
+            expected, oldresult = bracketstack[-1]
+            if not expected == char:
+                raise BracketUnexpectedCloseError(char)
+            del bracketstack[-1]
+            oldresult.append(result)
+            result = oldresult
+        elif not quoted and prefixes and char in prefixes and cur == [None]:
+            cur = [ParserPrefix(char)]
+            cur.append(None)
         else:
-            if cur is not None:
-                cur = cur + char
+            if len(cur):
+                if cur[-1] is None:
+                    cur[-1] = char
+                else:
+                    cur[-1] += char
             else:
-                curname = curname + char
+                cur.append(char)
             noquote = True
 
         idx += 1
 
-    if cur is None:
-        cur = curname
-        curname = None
-    cur_present = cur is not None
-    if not cur and not cur_quoted:
-        cur = None
-    if curname is not None:
-        keyword[curname] = cur
-    elif cur_present:
-        positional.append(cur)
+    if bracketstack:
+        raise BracketMissingCloseError(bracketstack[-1][0])
+
+    if quoted:
+        if len(cur):
+            if cur[-1] is None:
+                cur[-1] = quoted
+            else:
+                cur[-1] = quoted + cur[-1]
+        else:
+            cur.append(quoted)
+
+    additem(result, cur, separator_count, nextitemsep)
+
+    return result
+
+def parse_quoted_separated(args, separator=',', name_value=True, seplimit=0):
+    result = []
+    positional = result
+    if name_value:
+        name_value_separator = '='
+        trailing = []
+        keywords = {}
+    else:
+        name_value_separator = None
+
+    l = parse_quoted_separated_ext(args, separator=separator,
+                                   name_value_separator=name_value_separator,
+                                   seplimit=seplimit)
+    for item in l:
+        if isinstance(item, tuple):
+            key, value = item
+            if key is None:
+                key = u''
+            keywords[key] = value
+            positional = trailing
+        else:
+            positional.append(item)
 
     if name_value:
-        return ret_positional, keyword, ret_trailing
-    else:
-        return ret_positional
-
+        return result, keywords, trailing
+    return result
 
 def get_bool(request, arg, name=None, default=None):
     """
@@ -1418,11 +1575,11 @@ def get_bool(request, arg, name=None, default=None):
     else:
         if name:
             raise ValueError(
-                _('Argument "%s" must be a boolean value, not "%s"', formatted=False) % (
+                _('Argument "%s" must be a boolean value, not "%s"') % (
                     name, arg))
         else:
             raise ValueError(
-                _('Argument must be a boolean value, not "%s"', formatted=False) % arg)
+                _('Argument must be a boolean value, not "%s"') % arg)
 
 
 def get_int(request, arg, name=None, default=None):
@@ -1450,11 +1607,11 @@ def get_int(request, arg, name=None, default=None):
     except ValueError:
         if name:
             raise ValueError(
-                _('Argument "%s" must be an integer value, not "%s"', formatted=False) % (
+                _('Argument "%s" must be an integer value, not "%s"') % (
                     name, arg))
         else:
             raise ValueError(
-                _('Argument must be an integer value, not "%s"', formatted=False) % arg)
+                _('Argument must be an integer value, not "%s"') % arg)
 
 
 def get_float(request, arg, name=None, default=None):
@@ -1481,11 +1638,11 @@ def get_float(request, arg, name=None, default=None):
     except ValueError:
         if name:
             raise ValueError(
-                _('Argument "%s" must be a floating point value, not "%s"', formatted=False) % (
+                _('Argument "%s" must be a floating point value, not "%s"') % (
                     name, arg))
         else:
             raise ValueError(
-                _('Argument must be a floating point value, not "%s"', formatted=False) % arg)
+                _('Argument must be a floating point value, not "%s"') % arg)
 
 
 def get_complex(request, arg, name=None, default=None):
@@ -1514,11 +1671,11 @@ def get_complex(request, arg, name=None, default=None):
     except ValueError:
         if name:
             raise ValueError(
-                _('Argument "%s" must be a complex value, not "%s"', formatted=False) % (
+                _('Argument "%s" must be a complex value, not "%s"') % (
                     name, arg))
         else:
             raise ValueError(
-                _('Argument must be a complex value, not "%s"', formatted=False) % arg)
+                _('Argument must be a complex value, not "%s"') % arg)
 
 
 def get_unicode(request, arg, name=None, default=None):
@@ -1566,16 +1723,86 @@ def get_choice(request, arg, name=None, choices=[None]):
         _ = request.getText
         if name:
             raise ValueError(
-                _('Argument "%s" must be one of "%s", not "%s"', formatted=False) % (
+                _('Argument "%s" must be one of "%s", not "%s"') % (
                     name, '", "'.join(choices), arg))
         else:
             raise ValueError(
-                _('Argument must be one of "%s", not "%s"', formatted=False) % (
+                _('Argument must be one of "%s", not "%s"') % (
                     '", "'.join(choices), arg))
 
     return arg
 
 
+class IEFArgument:
+    """
+    Base class for new argument parsers for
+    invoke_extension_function.
+    """
+    def __init__(self):
+        pass
+
+    def parse_argument(self, s):
+        """
+        Parse the argument given in s (a string) and return
+        the argument for the extension function.
+        """
+        raise NotImplementedError
+
+    def get_default(self):
+        """
+        Return the default for this argument.
+        """
+        raise NotImplementedError
+
+
+class UnitArgument(IEFArgument):
+    """
+    Argument class for invoke_extension_function that forces
+    having any of the specified units given for a value.
+
+    Note that the default unit is "mm".
+
+    Use, for example, "UnitArgument('7mm', float, ['%', 'mm'])".
+
+    If the defaultunit parameter is given, any argument that
+    can be converted into the given argtype is assumed to have
+    the default unit. NOTE: This doesn't work with a choice
+    (tuple or list) argtype.
+    """
+    def __init__(self, default, argtype, units=['mm'], defaultunit=None):
+        """
+        Initialise a UnitArgument giving the default,
+        argument type and the permitted units.
+        """
+        IEFArgument.__init__(self)
+        self._units = list(units)
+        self._units.sort(lambda x, y: len(y) - len(x))
+        self._type = argtype
+        self._defaultunit = defaultunit
+        assert defaultunit is None or defaultunit in units
+        if default is not None:
+            self._default = self.parse_argument(default)
+        else:
+            self._default = None
+
+    def parse_argument(self, s):
+        for unit in self._units:
+            if s.endswith(unit):
+                ret = (self._type(s[:len(s) - len(unit)]), unit)
+                return ret
+        if self._defaultunit is not None:
+            try:
+                return (self._type(s), self._defaultunit)
+            except ValueError:
+                pass
+        units = ', '.join(self._units)
+        ## XXX: how can we translate this?
+        raise ValueError("Invalid unit in value %s (allowed units: %s)" % (s, units))
+
+    def get_default(self):
+        return self._default
+
+
 class required_arg:
     """
     Wrap a type in this class and give it as default argument
@@ -1588,7 +1815,7 @@ class required_arg:
         @param argtype: the type the argument should have
         """
         if not (argtype in (bool, int, long, float, complex, unicode) or
-                isinstance(argtype, (tuple, list))):
+                isinstance(argtype, (IEFArgument, tuple, list))):
             raise TypeError("argtype must be a valid type")
         self.argtype = argtype
 
@@ -1641,6 +1868,11 @@ def invoke_extension_function(request, function, args, fixed_args=[]):
             return get_float(request, value, name)
         elif default is complex:
             return get_complex(request, value, name)
+        elif isinstance(default, IEFArgument):
+            # defaults handled later
+            if value is None:
+                return None
+            return default.parse_argument(value)
         elif isinstance(default, required_arg):
             if isinstance(default.argtype, (tuple, list)):
                 # treat choice specially and return None if no choice
@@ -1722,13 +1954,13 @@ def invoke_extension_function(request, function, args, fixed_args=[]):
 
     if positional:
         if not allow_trailing:
-            raise ValueError(_('Too many arguments', formatted=False))
+            raise ValueError(_('Too many arguments'))
         trailing_args.extend(positional)
 
     if trailing_args:
         if not allow_trailing:
             raise ValueError(_('Cannot have arguments without name following'
-                               ' named arguments', formatted=False))
+                               ' named arguments'))
         kwargs['_trailing_args'] = trailing_args
 
     # type-convert all keyword arguments to the type
@@ -1741,9 +1973,11 @@ def invoke_extension_function(request, function, args, fixed_args=[]):
             # went wrong (if it does)
             kwargs[argname] = _convert_arg(request, kwargs[argname],
                                            defaults[argname], argname)
-            if (kwargs[argname] is None
-                and isinstance(defaults[argname], required_arg)):
-                raise ValueError(_('Argument "%s" is required', formatted=False) % argname)
+            if kwargs[argname] is None:
+                if isinstance(defaults[argname], required_arg):
+                    raise ValueError(_('Argument "%s" is required') % argname)
+                if isinstance(defaults[argname], IEFArgument):
+                    kwargs[argname] = defaults[argname].get_default()
 
         if not argname in argnames:
             # move argname into _kwargs parameter
@@ -1753,7 +1987,7 @@ def invoke_extension_function(request, function, args, fixed_args=[]):
     if kwargs_to_pass:
         kwargs['_kwargs'] = kwargs_to_pass
         if not allow_kwargs:
-            raise ValueError(_(u'No argument named "%s"', formatted=False) % (
+            raise ValueError(_(u'No argument named "%s"') % (
                 kwargs_to_pass.keys()[0]))
 
     return function(*fixed_args, **kwargs)
@@ -1799,7 +2033,7 @@ def parseAttributes(request, attrstring, endtoken=None, extension=None):
         # call extension function with the current token, the parser, and the dict
         if extension:
             found_flag, msg = extension(key, parser, attrs)
-            #request.log("%r = extension(%r, parser, %r)" % (msg, key, attrs))
+            #logging.debug("%r = extension(%r, parser, %r)" % (msg, key, attrs))
             if found_flag:
                 continue
             elif msg:
@@ -1812,7 +2046,7 @@ def parseAttributes(request, attrstring, endtoken=None, extension=None):
             msg = str(err)
             break
         if eq != "=":
-            msg = _('Expected "=" to follow "%(token)s"', formatted=False) % {'token': key}
+            msg = _('Expected "=" to follow "%(token)s"') % {'token': key}
             break
 
         try:
@@ -1821,7 +2055,7 @@ def parseAttributes(request, attrstring, endtoken=None, extension=None):
             msg = str(err)
             break
         if not val:
-            msg = _('Expected a value for key "%(token)s"', formatted=False) % {'token': key}
+            msg = _('Expected a value for key "%(token)s"') % {'token': key}
             break
 
         key = escape(key) # make sure nobody cheats
@@ -2076,6 +2310,22 @@ def isStrictWikiname(name, word_re=re.compile(ur"^(?:[%(u)s][%(l)s]+){2,}$" % {'
     return word_re.match(name)
 
 
+def is_URL(arg, schemas=config.url_schemas):
+    """ Return True if arg is a URL (with a schema given in the schemas list).
+
+        Note: there are not that many requirements for generic URLs, basically
+        the only mandatory requirement is the ':' between schema and rest.
+        Schema itself could be anything, also the rest (but we only support some
+        schemas, as given in config.url_schemas, so it is a bit less ambiguous).
+    """
+    if ':' not in arg:
+        return False
+    for schema in schemas:
+        if arg.startswith(schema + ':'):
+            return True
+    return False
+
+
 def isPicture(url):
     """
     Is this a picture's url?
@@ -2084,8 +2334,8 @@ def isPicture(url):
     @rtype: bool
     @return: true if url points to a picture
     """
-    extpos = url.rfind(".")
-    return extpos > 0 and url[extpos:].lower() in ['.gif', '.jpg', '.jpeg', '.png', '.bmp', '.ico', ]
+    extpos = url.rfind(".") + 1
+    return extpos > 1 and url[extpos:].lower() in config.browser_supported_images
 
 
 def link_tag(request, params, text=None, formatter=None, on=None, **kw):
@@ -2137,7 +2387,7 @@ def link_tag(request, params, text=None, formatter=None, on=None, **kw):
             tag = '<a%s href="%s/%s">' % (attrs, request.getScriptname(), params)
             if not on:
                 tag = "%s%s</a>" % (tag, text)
-        request.log("Warning: wikiutil.link_tag called without formatter and without request.html_formatter. tag=%r" % (tag, ))
+        logging.warning("wikiutil.link_tag called without formatter and without request.html_formatter. tag=%r" % (tag, ))
     return tag
 
 def containsConflictMarker(text):
@@ -2181,15 +2431,16 @@ def anchor_name_from_text(text):
 ########################################################################
 
 def createTicket(request, tm=None, action=None):
-    """ Create a ticket using a site-specific secret (the config)
-    
+    """ Create a ticket using a configured secret
+
         @param tm: unix timestamp (optional, uses current time if not given)
         @param action: action name (optional, uses current action if not given)
                        Note: if you create a ticket for a form that calls another
                              action than the current one, you MUST specify the
                              action you call when posting the form.
     """
-    import sha
+
+    from MoinMoin.support.python_compatibility import hash_new
     if tm is None:
         tm = "%010x" % time.time()
 
@@ -2205,20 +2456,12 @@ def createTicket(request, tm=None, action=None):
         except:
             action = 'None'
 
+    secret = request.cfg.secrets['wikiutil/tickets']
+    digest = hash_new('sha1', secret)
 
     ticket = "%s.%s.%s" % (tm, pagename, action)
-    digest = sha.new()
     digest.update(ticket)
 
-    varnames = ['data_dir', 'data_underlay_dir', 'language_default',
-                'mail_smarthost', 'mail_from', 'page_front_page',
-                'theme_default', 'sitename', 'logo_string',
-                'interwikiname', 'user_homewiki', 'acl_rights_before', ]
-    for varname in varnames:
-        var = getattr(request.cfg, varname, None)
-        if isinstance(var, (str, unicode)):
-            digest.update(repr(var))
-
     return "%s.%s" % (ticket, digest.hexdigest())
 
 
@@ -2241,7 +2484,7 @@ def checkTicket(request, ticket):
     return ticket == ourticket
 
 
-def renderText(request, Parser, text, line_anchors=False):
+def renderText(request, Parser, text):
     """executes raw wiki markup with all page elements"""
     import StringIO
     out = StringIO.StringIO()
